package net.awired.ajsl.persistence.entity;

import java.io.Serializable;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import com.google.common.base.Objects;

@MappedSuperclass
// none because jaxb cannot handle Serializable generics
// to serialize id override getId() with @XmlElement
@XmlAccessorType(XmlAccessType.NONE)
public abstract class IdEntityImpl<KEY_TYPE extends Serializable> implements IdEntity<KEY_TYPE> {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue
    protected KEY_TYPE id;

    ///////////////////////////////////////////////////////////////////////////

    @Override
    public String toString() {
        return this.getClass().getName() + "[id=" + this.getId() + "]";
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(id);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof IdEntityImpl) {
            @SuppressWarnings("unchecked")
            IdEntityImpl<KEY_TYPE> other = (IdEntityImpl<KEY_TYPE>) obj;
            return Objects.equal(id, other.id);
        } else {
            return false;
        }
    }

    // ////////////////////////////////////////////////////////////////////////

    @Override
    public KEY_TYPE getId() {
        return id;
    }

    @Override
    public void setId(KEY_TYPE id) {
        this.id = id;
    }
}
